/**
 * Class for dealing with static scoring.
 */

package org.ucam.srcf.assassins.strawberry;

import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;


import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

class KirikaSparse implements StaticScoringAlgorithm {
   private double A[][];
   private double deaths[];
   private double x[];
   private Map assIndex=new HashMap();
   private int next_index=0;
   private boolean dirtyMatrix=false;
   private static final int c = 6;
   private static final double tol = 0.0001;

   private Map innocents;
   private Map bonuses;

   private static native void realAddDeath(int killer, int victim);
   private static native void realinit(int n);
   private static native void realfinal();
   private static native void realsolve(double[] u, int c, double tol);

   public void init (int n) {
      x = new double [n];
      for(int i=0; i<n; i++)
         x[i]=1;
      realinit(n);
      innocents = new HashMap();
      bonuses = new HashMap();
   }

   /** 
    * Wrapper around the munge and solve routine.
    * @param c Scoring constant.
    * @return Map from AssassinViewer to Double representing score.
    */
   public Map score() {
      realsolve(x, c, tol);

      Map scores=new HashMap();
      Iterator i=assIndex.keySet().iterator();
      while(i.hasNext()) {
         AssassinViewer current=(AssassinViewer) i.next();
         int index=getIndex(current);
         double modifier = 
            getInnocentPenalty(getIntFromMap(innocents, current, 0)) +
            getDoubleFromMap(bonuses, current, 0);
         long int_score = Math.round(100*(x[index]+modifier));
         scores.put(current, new Double(int_score));
      }

      return scores;
   }

   private void printMatrix(int n) {
      for(int i=0; i<n; i++)
         printVector(A[i], n);
   }

   private void printVector(double[] x, int n) {
      String str=x[0]+"";
      for(int i=1; i<n; i++)
         str+=", "+x[i];
      Debug.print(this, str);
   }
   
   public void addDeath(Death d) throws AssassinSQLException {
      if(dirtyMatrix)
         throw new RuntimeException("Matrix already scored!");
      
      int kindex=getIndex(d.getKiller());
      int vindex=getIndex(d.getVictim());

      realAddDeath(kindex,vindex);
   }

   private int getIndex(AssassinViewer a) {
      Integer index = (Integer) assIndex.get(a);
      if(index==null)
         assIndex.put(a, index = new Integer(next_index++));
      return index.intValue();
   }

   /**
    * Tells the algorithm about an innocent death(s).
    * It is up to the alogrithm to use it as it sees fit
    * @param who Who killed it/them?
    * @param count How many they killed.
    */
   public void reportInnocents(AssassinViewer who, int count) throws AssassinSQLException {
      int innos = getIntFromMap(innocents, who, 0);
      innos+=count;
      innocents.put(who, new Integer(innos));
   }

   /**
    * Tells the algorithm about bonuses.
    * It is up to the algorithm to actually apply them
    * @param who Who gets the bonus.
    * @param value How much is it worth?
    */
   public void giveBonus(AssassinViewer who, double value) throws AssassinSQLException {
      double bonus = getDoubleFromMap(bonuses, who, 0);
      bonus+=value;
      bonuses.put(who, new Double(bonus));
   }

   static double getInnocentPenalty(int count) {
      double penalty=0;
      for(int i=1; i<=count; i++)
         penalty += 0.1 * i;
      return penalty;
   }

   private double getDoubleFromMap(Map m, Object key, double defaultValue) {
      Double valueD = (Double) m.get(key);
      double value = defaultValue;
      if(valueD != null)
         value = valueD.doubleValue();
      return value;
   }

   private int getIntFromMap(Map m, Object key, int defaultValue) {
      Integer valueI = (Integer) m.get(key);
      int value = defaultValue;
      if(valueI != null)
         value = valueI.intValue();
      return value;
   }
}
